import logging

from twistit.lib.base import BaseController, render, redirect_to, c, request, response
from twistit.model import Game, Block, Session
from twistit.lib.generation import generate_field, shuffle
from twistit.lib.solver import Solver

log = logging.getLogger(__name__)

from paste.debug.profile import profile_decorator

class GameController(BaseController):

    def index(self):
        return render('/game.mako')

    def new(self):
        return render('/new.mako')
        

    def create(self):
        #, field_size, error_margin, sides, jokers)
        get = request.params.get

        size = map(int, get('field-size').split('x'))
        error_margin = int(get('error-margin'))
        sides = map(int,get('sides').split(','))
        jokers = int(get('jokers'))


        field = generate_field(size, sides=sides, jokers=jokers)
        steps = shuffle(field)

        game = Game(field)
        game.maximum_steps = steps + error_margin
        

        Session.save(field)
        Session.save(game)
        Session.commit()

        return redirect_to(action='play', id=str(game.id))

    def play(self, id, **kwargs):
        game = Session.query(Game).get(id)
        c.game = game


        field = game.field.light_weight_copy()

        return render('/play.mako')
        
    def rotate(self, blockid, direction):
        angle_mapping = {'left': 270, 'right': 90}

        block = Session.query(Block).get(blockid)
        block.rotate(angle_mapping[direction])

        field = block.field
        game = field.game[0]
        game.steps_made += 1
        c.game = game

        Session.save(block)
        Session.commit()

        c.block = block

        response.headers.add('x-is-solved', str(field.is_solved()))
        return render('/update_block.mako')
